df = read.csv("temperatures.csv")

temp = df$Value

model = arima(temp, order = c(4,1,4))
model

plot(temp, type = "l", main = "Global temperature differences", 
     ylab = "temperature difference",  lwd = 2)
lines(temp - model$residuals, col = "red", lwd = 2)
legend("bottomleft", c("data", "ARIMA(4,1,4) model"), fill = c("black", "red"))

# first order diffence
plot(diff(temp), type = "l", lwd = 2, main = "Yearly difference for temperature", ylab = "difference")
grid()


# optimal model
P = c(1,2,3,4,5,6,7,8)
Q = c(1,2,3,4,5,6,7,8)
d = 1

AIC_mat = matrix(0, nrow = length(P), ncol = length(Q))

for (p in P){
  for (q in Q){
    model = arima(temp, order = c(p, d, q))
    AIC_mat[p, q] = model$aic
  }
} 

heatmap(AIC_mat, Colv = NA, Rowv = NA, scale="column", xlab = "q", ylab = "p")


# predict
temp_ts = ts(temp)
temp_train = ts(temp_ts[1:100])
temp_test = ts(temp_ts[101:107], start = 101)
model_opt = arima(temp_train, order = c(4,1,4))


pred_ts = predict((model_opt), n.ahead = 10)

plot(temp_train, xlim = c(0, 107), ylim = c(-1, 2), lwd =2, main ="ARIMA(4,1,4) model for temperature differences")
lines(temp_test, col = "blue", lwd=2)
lines(pred_ts$pred, col = "red", lwd=2)
lines(pred_ts$pred + 2*pred_ts$se, col = "red", lty = 2)
lines(pred_ts$pred - 2*pred_ts$se, col = "red", lty = 2)
legend("topleft", c("truth", "predicted"), fill = c("blue", "red"))
grid()
